const Lang = imports.lang;
const Signals = imports.signals;

const Atk = imports.gi.Atk;
const Soup = imports.gi.Soup;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const GLib = imports.gi.GLib
const Gio = imports.gi.Gio;

const Main = imports.ui.main;
const Overview = imports.ui.overview;
const ExtensionSystem = imports.ui.extensionSystem;
const PopupMenu = imports.ui.popupMenu;

const ExtensionUtils  = imports.misc.extensionUtils;
const Params = imports.misc.params;

//-----------------------------------------------------------------------------------------

const ExtensionItem = new Lang.Class({
	Name: 'ExtensionItem',

	_init: function(params) {
		params = Params.parse(params, {
			extension: null,
			id: null
		});

		if(!params.extension || !params.id) {
			throw new Error("Must provide extension and extension ID.");
		}

		this.id = params.id;
		this.extension = params.extension;


		this.title = this.extension.metadata.name;
		this.description = this.extension.metadata.description;

		this.error = this.extension.state == 3; // ERROR
		this.enabled = this.extension.state == 1; // ENABLED
		
		this._textBox = new St.BoxLayout({
			vertical: true,
		});


		this._label =  new St.Label({
			text: GLib.markup_escape_text(this.title, -1),
			style_class: 'extension-title'
		});

		this._description = new St.Label({
			text: GLib.markup_escape_text(this.description, -1),
			style_class: 'extension-desc'
		});

		this._url = new St.Button({
			label: "Launch this extension's website!",
			style_class: 'extension-website',
			x_align: St.Align.START
		});

		this._url.connect('button-release-event', 
				Lang.bind(this, this._urlClicked));

		this._fallback = new St.Label({
			text: 'This extension does not have an associated website.',
			style_class: 'extension-no-website'
		});
		
		this._textBox.add(this._label);
		this._textBox.add(this._description);

		if(this.extension.metadata.url)
			this._textBox.add(this._url);
		else
			this._textBox.add(this._fallback);

		this._switch = new PopupMenu.Switch(this.enabled);

		this._switch.actor.reactive = true;

		this._switch.actor.connect('button-release-event', 
				Lang.bind(this, this._onToggleSwitch));

		this._errorBox = new St.Bin({
			style_class: 'extension-error'
		})

		this.actor = new St.BoxLayout({
			accessible_role: Atk.Role.LIST_ITEM,
			style_class: 'extension'
		});

		this.actor.add(this._textBox, { expand: true });
		this.actor.add(this._errorBox);
		this.actor.add(this._switch.actor);

	},

	destroy: function() {
		// Nothing to do.
	},

	_onToggleSwitch: function(actor, event) {
		if(this.enabled) {
			global.log('Disabling ' + this.id);
			ExtensionSystem.disableExtension(this.id);
		} else {
			global.log('Enabling ' + this.id);
			ExtensionSystem.enableExtension(this.id);
		}

	},

	_urlClicked: function(actor, event) {
		let url = this.extension.metadata.url;
		Gio.app_info_launch_default_for_uri(url, global.create_app_launch_context());

		Main.overview.hide();
	},

	stateChanged: function(newState) {
		this.enabled = newState == 1;
		this.error = newState == 3;

		this._switch.setToggleState(this.enabled);
	}
});

//-----------------------------------------------------------------------------------------

const ExtensionList = new Lang.Class({
	Name: 'ExtensionList',
	
	_init: function() {

		this._list = new St.BoxLayout({
			vertical: true,
			reactive: true,
			accessible_role: Atk.Role.LIST
		});
		
		this.actor = new St.ScrollView({
			x_fill: false,
			y_fill: false,
			style_class: 'vfade'
		});

		//this._wid = Main.intializeDeferredWork(this.actor, Lang.bind(this, this.refresh));

		this.extensions = []
		
        this.actor.add_actor(this._list);

		this._handler = ExtensionSystem.connect('extension-loaded', 
				Lang.bind(this, this._extLoaded));

		this._otherHandler = ExtensionSystem.connect('extension-state-changed',
				Lang.bind(this, this._extStateChanged));

		this._otherOtherHandler = this.actor.connect('notify::mapped',
				Lang.bind(this, this._notifyMapped));
	},

	destroy: function() {
		this.actor.destroy();

		ExtensionSystem.disconnect(this._handler);
		ExtensionSystem.disconnect(this._otherHandler);
		// Unregister the Deferred Work.
	},

	installMore: function() {
		let url = 'https://extensions.gnome.org/';
		Gio.app_info_launch_default_for_uri(url, global.create_app_launch_context());

		Main.overview.hide();
	},

	_extLoaded: function(source, id) {
		this._addExtension(id, ExtensionUtils.extensions[id]);	
	},

	_extStateChanged: function(source, extension) {
		let ext = this._findExtension(extension.uuid);
		if(ext)
			ext.stateChanged(extension.state);
	},

	_findExtension: function(id) {
		for(let i in this.extensions) {
			let ext = this.extensions[i];
			if(ext.id == id)
				return ext
		}
		return null;
	},

	_addExtension: function(id, ext) {
		if(!ext || this._findExtension(id)) {
			return;
		}

		let ext_item = new ExtensionItem({
			id: id,
			extension: ext
		});

		this.extensions.push(ext_item);

		this._list.add(ext_item.actor, { expand: false, x_fill: true, y_fill: false });
		
		this.actor.queue_relayout();
	},
	
	_notifyMapped: function() {
		if(this.actor.mapped)
			this.loadData();
	},

	loadData: function() {
		this.extensions = [];
		this._list.destroy_all_children();

		let installBtn = new St.Button({
			label: 'Get Extensions!',
			style_class: 'get-extensions'
		});
		
		installBtn.connect('button-release-event',
				Lang.bind(this, this.installMore));
			
		this._list.add(installBtn);

		for(let id in ExtensionUtils.extensions) {
			this._addExtension(id, ExtensionUtils.extensions[id]);
		}

	}
});

Signals.addSignalMethods(ExtensionList.prototype);

//-----------------------------------------------------------------------------------------

let ext_list = null;

let ext_tab = null;
let ext_index = -1;

function init(meta) {

}

function enable() {
	ext_list = new ExtensionList();

	Main.overview._viewSelector.
		addViewTab('extensions', 'Extensions', ext_list.actor, 'extensions');
	
	ext_index = Main.overview._viewSelector._tabs.length - 1;
	ext_tab = Main.overview._viewSelector._tabs[ext_index];
}

function disable() {	
	Main.overview._viewSelector._tabBox.remove_actor(ext_tab.title);
	Main.overview._viewSelector._pageArea.remove_actor(ext_tab.page);
	Main.overview._viewSelector._tabs.splice(ext_index, ext_index);

	ext_list.destroy();
	ext_list = null;
}
